import { decorated_getter_flag } from './entrance-decorator.js';
export async function up(entrances, arg = ['*'], ref) {
    const { includes, excludes } = Array.isArray(arg)
        ? { includes: arg, excludes: [] }
        : arg;
    const includeMatchers = includes.map(include => buildMatcher(include));
    const excludeMatchers = excludes.map(exclude => buildMatcher(exclude));
    const entryPromises = [];
    const keys = getPrototypes(entrances)
        .flatMap(prototype => Object.entries(Object.getOwnPropertyDescriptors(prototype)).filter(([_name, { get }]) => (get ? get[decorated_getter_flag] : false)))
        .map(([name]) => name);
    for (const key of keys) {
        if (!includeMatchers.some(matcher => matcher(key)) ||
            excludeMatchers.some(matcher => matcher(key))) {
            continue;
        }
        entryPromises.push(Promise.resolve(entrances[key]).then(value => [
            key,
            {
                get() {
                    return value;
                },
            },
        ]));
    }
    const entries = await Promise.all(entryPromises);
    if (ref) {
        Object.setPrototypeOf(ref, entrances);
        for (const [key, descriptor] of entries) {
            Object.defineProperty(ref, key, descriptor);
        }
        return ref;
    }
    else {
        return Object.create(entrances, Object.fromEntries(entries));
    }
}
function buildMatcher(pattern) {
    const starIndex = pattern.indexOf('*');
    if (starIndex < 0) {
        return key => key === pattern;
    }
    else {
        const prefix = pattern.slice(0, starIndex);
        const postfix = pattern.slice(starIndex + 1);
        return key => key.startsWith(prefix) && key.endsWith(postfix);
    }
}
function getPrototypes(object) {
    const prototypes = [];
    // eslint-disable-next-line no-cond-assign
    while ((object = Object.getPrototypeOf(object))) {
        if (object === Object.prototype) {
            break;
        }
        prototypes.push(object);
    }
    return prototypes;
}
//# sourceMappingURL=up.js.map